# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'pyxpad_main.ui'
#
# Created: Fri Oct  7 15:25:29 2016
#      by: pyside-uic 0.2.15 running on PySide 1.2.4
#
# WARNING! All changes made in this file will be lost!

from PySide import QtCore, QtGui

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(803, 659)
        self.centralwidget = QtGui.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_2 = QtGui.QGridLayout(self.centralwidget)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.tabWidget = QtGui.QTabWidget(self.centralwidget)
        self.tabWidget.setObjectName("tabWidget")
        self.sourceTab = QtGui.QWidget()
        self.sourceTab.setObjectName("sourceTab")
        self.gridLayout_3 = QtGui.QGridLayout(self.sourceTab)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.gridLayout = QtGui.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.sourceDescription = QtGui.QCheckBox(self.sourceTab)
        self.sourceDescription.setObjectName("sourceDescription")
        self.gridLayout.addWidget(self.sourceDescription, 0, 6, 1, 1)
        self.shotLabel = QtGui.QLabel(self.sourceTab)
        self.shotLabel.setObjectName("shotLabel")
        self.gridLayout.addWidget(self.shotLabel, 0, 0, 1, 1)
        self.tracePattern = QtGui.QLineEdit(self.sourceTab)
        self.tracePattern.setObjectName("tracePattern")
        self.gridLayout.addWidget(self.tracePattern, 0, 5, 1, 1)
        self.shotInput = QtGui.QLineEdit(self.sourceTab)
        self.shotInput.setObjectName("shotInput")
        self.gridLayout.addWidget(self.shotInput, 0, 1, 1, 1)
        self.readDataButton = QtGui.QPushButton(self.sourceTab)
        self.readDataButton.setObjectName("readDataButton")
        self.gridLayout.addWidget(self.readDataButton, 0, 2, 1, 1)
        self.traceLabel = QtGui.QLabel(self.sourceTab)
        self.traceLabel.setObjectName("traceLabel")
        self.gridLayout.addWidget(self.traceLabel, 0, 4, 1, 1)
        self.lastShotButton = QtGui.QPushButton(self.sourceTab)
        self.lastShotButton.setObjectName("lastShotButton")
        self.gridLayout.addWidget(self.lastShotButton, 0, 3, 1, 1)
        self.gridLayout_3.addLayout(self.gridLayout, 0, 0, 1, 1)
        self.splitter = QtGui.QSplitter(self.sourceTab)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.treeView = QtGui.QTreeWidget(self.splitter)
        self.treeView.setColumnCount(1)
        self.treeView.setObjectName("treeView")
        self.treeView.headerItem().setText(0, "Source")
        self.treeView.header().setVisible(False)
        self.treeView.header().setDefaultSectionSize(200)
        self.treeView.header().setStretchLastSection(True)
        self.sourceTable = QtGui.QTableWidget(self.splitter)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.sourceTable.sizePolicy().hasHeightForWidth())
        self.sourceTable.setSizePolicy(sizePolicy)
        self.sourceTable.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.sourceTable.setAlternatingRowColors(True)
        self.sourceTable.setSelectionMode(QtGui.QAbstractItemView.MultiSelection)
        self.sourceTable.setShowGrid(False)
        self.sourceTable.setColumnCount(1)
        self.sourceTable.setObjectName("sourceTable")
        self.sourceTable.setColumnCount(1)
        self.sourceTable.setRowCount(0)
        self.sourceTable.horizontalHeader().setVisible(False)
        self.sourceTable.horizontalHeader().setDefaultSectionSize(200)
        self.sourceTable.horizontalHeader().setStretchLastSection(False)
        self.sourceTable.verticalHeader().setVisible(False)
        self.sourceTable.verticalHeader().setDefaultSectionSize(20)
        self.gridLayout_3.addWidget(self.splitter, 1, 0, 1, 1)
        self.tabWidget.addTab(self.sourceTab, "")
        self.dataTab = QtGui.QWidget()
        self.dataTab.setObjectName("dataTab")
        self.gridLayout_5 = QtGui.QGridLayout(self.dataTab)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.splitter_2 = QtGui.QSplitter(self.dataTab)
        self.splitter_2.setOrientation(QtCore.Qt.Vertical)
        self.splitter_2.setObjectName("splitter_2")
        self.dataTable = QtGui.QTableWidget(self.splitter_2)
        self.dataTable.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.dataTable.setAlternatingRowColors(True)
        self.dataTable.setSelectionMode(QtGui.QAbstractItemView.MultiSelection)
        self.dataTable.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
        self.dataTable.setShowGrid(False)
        self.dataTable.setGridStyle(QtCore.Qt.SolidLine)
        self.dataTable.setObjectName("dataTable")
        self.dataTable.setColumnCount(4)
        self.dataTable.setRowCount(0)
        item = QtGui.QTableWidgetItem()
        self.dataTable.setHorizontalHeaderItem(0, item)
        item = QtGui.QTableWidgetItem()
        self.dataTable.setHorizontalHeaderItem(1, item)
        item = QtGui.QTableWidgetItem()
        self.dataTable.setHorizontalHeaderItem(2, item)
        item = QtGui.QTableWidgetItem()
        self.dataTable.setHorizontalHeaderItem(3, item)
        self.dataTable.horizontalHeader().setStretchLastSection(True)
        self.dataTable.verticalHeader().setDefaultSectionSize(20)
        self.textOutput = QtGui.QTextEdit(self.splitter_2)
        self.textOutput.setReadOnly(True)
        self.textOutput.setObjectName("textOutput")
        self.layoutWidget = QtGui.QWidget(self.splitter_2)
        self.layoutWidget.setObjectName("layoutWidget")
        self.gridLayout_4 = QtGui.QGridLayout(self.layoutWidget)
        self.gridLayout_4.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.label_3 = QtGui.QLabel(self.layoutWidget)
        self.label_3.setObjectName("label_3")
        self.gridLayout_4.addWidget(self.label_3, 0, 0, 1, 1)
        self.commandInput = ConsoleWidget(self.layoutWidget)
        self.commandInput.setObjectName("commandInput")
        self.gridLayout_4.addWidget(self.commandInput, 0, 1, 1, 1)
        self.commandButton = QtGui.QPushButton(self.layoutWidget)
        self.commandButton.setObjectName("commandButton")
        self.gridLayout_4.addWidget(self.commandButton, 0, 2, 1, 1)
        self.gridLayout_5.addWidget(self.splitter_2, 0, 0, 1, 1)
        self.tabWidget.addTab(self.dataTab, "")
        self.plotTab = QtGui.QWidget()
        self.plotTab.setObjectName("plotTab")
        self.tabWidget.addTab(self.plotTab, "")
        self.gridLayout_2.addWidget(self.tabWidget, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 803, 19))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtGui.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuAdd_source = QtGui.QMenu(self.menuFile)
        self.menuAdd_source.setObjectName("menuAdd_source")
        self.menuPlot = QtGui.QMenu(self.menubar)
        self.menuPlot.setObjectName("menuPlot")
        self.menuCommand = QtGui.QMenu(self.menubar)
        self.menuCommand.setObjectName("menuCommand")
        self.menuHelp = QtGui.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtGui.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionLoadState = QtGui.QAction(MainWindow)
        self.actionLoadState.setObjectName("actionLoadState")
        self.actionSaveState = QtGui.QAction(MainWindow)
        self.actionSaveState.setObjectName("actionSaveState")
        self.actionExit = QtGui.QAction(MainWindow)
        self.actionExit.setObjectName("actionExit")
        self.actionXPAD_tree = QtGui.QAction(MainWindow)
        self.actionXPAD_tree.setObjectName("actionXPAD_tree")
        self.actionNetCDF_file = QtGui.QAction(MainWindow)
        self.actionNetCDF_file.setObjectName("actionNetCDF_file")
        self.actionPlot = QtGui.QAction(MainWindow)
        self.actionPlot.setObjectName("actionPlot")
        self.actionOPlot = QtGui.QAction(MainWindow)
        self.actionOPlot.setObjectName("actionOPlot")
        self.actionMPlot = QtGui.QAction(MainWindow)
        self.actionMPlot.setObjectName("actionMPlot")
        self.actionXYPlot = QtGui.QAction(MainWindow)
        self.actionXYPlot.setObjectName("actionXYPlot")
        self.actionZPlot = QtGui.QAction(MainWindow)
        self.actionZPlot.setObjectName("actionZPlot")
        self.actionBOUT_data = QtGui.QAction(MainWindow)
        self.actionBOUT_data.setObjectName("actionBOUT_data")
        self.actionContour = QtGui.QAction(MainWindow)
        self.actionContour.setObjectName("actionContour")
        self.actionContour_filled = QtGui.QAction(MainWindow)
        self.actionContour_filled.setObjectName("actionContour_filled")
        self.actionClearFig = QtGui.QAction(MainWindow)
        self.actionClearFig.setObjectName("actionClearFig")
        self.actionWrite_ASCII = QtGui.QAction(MainWindow)
        self.actionWrite_ASCII.setObjectName("actionWrite_ASCII")
        self.actionAdd = QtGui.QAction(MainWindow)
        self.actionAdd.setObjectName("actionAdd")
        self.actionSubtract = QtGui.QAction(MainWindow)
        self.actionSubtract.setObjectName("actionSubtract")
        self.actionMultiply = QtGui.QAction(MainWindow)
        self.actionMultiply.setObjectName("actionMultiply")
        self.actionDivide = QtGui.QAction(MainWindow)
        self.actionDivide.setObjectName("actionDivide")
        self.actionChop = QtGui.QAction(MainWindow)
        self.actionChop.setObjectName("actionChop")
        self.actionIntegrate = QtGui.QAction(MainWindow)
        self.actionIntegrate.setObjectName("actionIntegrate")
        self.actionDf_dt = QtGui.QAction(MainWindow)
        self.actionDf_dt.setObjectName("actionDf_dt")
        self.actionSmooth = QtGui.QAction(MainWindow)
        self.actionSmooth.setObjectName("actionSmooth")
        self.actionLow_pass_filter = QtGui.QAction(MainWindow)
        self.actionLow_pass_filter.setObjectName("actionLow_pass_filter")
        self.actionHigh_pass_filter = QtGui.QAction(MainWindow)
        self.actionHigh_pass_filter.setObjectName("actionHigh_pass_filter")
        self.actionBand_pass_filter = QtGui.QAction(MainWindow)
        self.actionBand_pass_filter.setObjectName("actionBand_pass_filter")
        self.actionFFTP = QtGui.QAction(MainWindow)
        self.actionFFTP.setObjectName("actionFFTP")
        self.actionRunFFT = QtGui.QAction(MainWindow)
        self.actionRunFFT.setObjectName("actionRunFFT")
        self.actionReciprocal = QtGui.QAction(MainWindow)
        self.actionReciprocal.setObjectName("actionReciprocal")
        self.actionExponential = QtGui.QAction(MainWindow)
        self.actionExponential.setObjectName("actionExponential")
        self.actionAbsolute = QtGui.QAction(MainWindow)
        self.actionAbsolute.setObjectName("actionAbsolute")
        self.actionArctan = QtGui.QAction(MainWindow)
        self.actionArctan.setObjectName("actionArctan")
        self.actionNlog = QtGui.QAction(MainWindow)
        self.actionNlog.setObjectName("actionNlog")
        self.actionNorm = QtGui.QAction(MainWindow)
        self.actionNorm.setObjectName("actionNorm")
        self.actionInvert = QtGui.QAction(MainWindow)
        self.actionInvert.setObjectName("actionInvert")
        self.actionAddCon = QtGui.QAction(MainWindow)
        self.actionAddCon.setObjectName("actionAddCon")
        self.actionSubCon = QtGui.QAction(MainWindow)
        self.actionSubCon.setObjectName("actionSubCon")
        self.actionMulCon = QtGui.QAction(MainWindow)
        self.actionMulCon.setObjectName("actionMulCon")
        self.actionDivCon = QtGui.QAction(MainWindow)
        self.actionDivCon.setObjectName("actionDivCon")
        self.actionPowCon = QtGui.QAction(MainWindow)
        self.actionPowCon.setObjectName("actionPowCon")
        self.actionChangeName = QtGui.QAction(MainWindow)
        self.actionChangeName.setObjectName("actionChangeName")
        self.actionChangeUnits = QtGui.QAction(MainWindow)
        self.actionChangeUnits.setObjectName("actionChangeUnits")
        self.actionClip = QtGui.QAction(MainWindow)
        self.actionClip.setObjectName("actionClip")
        self.actionStats = QtGui.QAction(MainWindow)
        self.actionStats.setObjectName("actionStats")
        self.actionTimeOff = QtGui.QAction(MainWindow)
        self.actionTimeOff.setObjectName("actionTimeOff")
        self.actionAbout = QtGui.QAction(MainWindow)
        self.actionAbout.setObjectName("actionAbout")
        self.actionDeleteTrace = QtGui.QAction(MainWindow)
        self.actionDeleteTrace.setObjectName("actionDeleteTrace")
        self.menuAdd_source.addAction(self.actionNetCDF_file)
        self.menuAdd_source.addAction(self.actionXPAD_tree)
        self.menuAdd_source.addAction(self.actionBOUT_data)
        self.menuFile.addAction(self.menuAdd_source.menuAction())
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionLoadState)
        self.menuFile.addAction(self.actionSaveState)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionWrite_ASCII)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionExit)
        self.menuPlot.addAction(self.actionPlot)
        self.menuPlot.addAction(self.actionOPlot)
        self.menuPlot.addAction(self.actionMPlot)
        self.menuPlot.addAction(self.actionZPlot)
        self.menuPlot.addAction(self.actionXYPlot)
        self.menuPlot.addAction(self.actionContour)
        self.menuPlot.addAction(self.actionContour_filled)
        self.menuPlot.addAction(self.actionClearFig)
        self.menuCommand.addAction(self.actionDeleteTrace)
        self.menuCommand.addSeparator()
        self.menuCommand.addAction(self.actionChop)
        self.menuCommand.addSeparator()
        self.menuCommand.addAction(self.actionIntegrate)
        self.menuCommand.addAction(self.actionDf_dt)
        self.menuCommand.addAction(self.actionSmooth)
        self.menuCommand.addAction(self.actionBand_pass_filter)
        self.menuCommand.addSeparator()
        self.menuCommand.addAction(self.actionAdd)
        self.menuCommand.addAction(self.actionSubtract)
        self.menuCommand.addAction(self.actionMultiply)
        self.menuCommand.addAction(self.actionDivide)
        self.menuCommand.addSeparator()
        self.menuCommand.addAction(self.actionFFTP)
        self.menuCommand.addAction(self.actionRunFFT)
        self.menuCommand.addSeparator()
        self.menuCommand.addAction(self.actionReciprocal)
        self.menuCommand.addAction(self.actionExponential)
        self.menuCommand.addAction(self.actionAbsolute)
        self.menuCommand.addAction(self.actionArctan)
        self.menuCommand.addAction(self.actionNlog)
        self.menuCommand.addAction(self.actionNorm)
        self.menuCommand.addAction(self.actionInvert)
        self.menuCommand.addAction(self.actionAddCon)
        self.menuCommand.addAction(self.actionSubCon)
        self.menuCommand.addAction(self.actionMulCon)
        self.menuCommand.addAction(self.actionDivCon)
        self.menuCommand.addAction(self.actionPowCon)
        self.menuCommand.addAction(self.actionChangeName)
        self.menuCommand.addAction(self.actionChangeUnits)
        self.menuCommand.addAction(self.actionClip)
        self.menuCommand.addAction(self.actionStats)
        self.menuCommand.addAction(self.actionTimeOff)
        self.menuHelp.addAction(self.actionAbout)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuPlot.menuAction())
        self.menubar.addAction(self.menuCommand.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QtGui.QApplication.translate("MainWindow", "PyXPad", None, QtGui.QApplication.UnicodeUTF8))
        self.sourceDescription.setText(QtGui.QApplication.translate("MainWindow", "Description", None, QtGui.QApplication.UnicodeUTF8))
        self.shotLabel.setText(QtGui.QApplication.translate("MainWindow", "Shot:", None, QtGui.QApplication.UnicodeUTF8))
        self.readDataButton.setText(QtGui.QApplication.translate("MainWindow", "&Read", None, QtGui.QApplication.UnicodeUTF8))
        self.traceLabel.setText(QtGui.QApplication.translate("MainWindow", "Trace:", None, QtGui.QApplication.UnicodeUTF8))
        self.lastShotButton.setToolTip(QtGui.QApplication.translate("MainWindow", "Get last shot number", None, QtGui.QApplication.UnicodeUTF8))
        self.lastShotButton.setText(QtGui.QApplication.translate("MainWindow", "&Last shot", None, QtGui.QApplication.UnicodeUTF8))
        self.sourceTable.setToolTip(QtGui.QApplication.translate("MainWindow", "List of available signals", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.sourceTab), QtGui.QApplication.translate("MainWindow", "&Sources", None, QtGui.QApplication.UnicodeUTF8))
        self.dataTable.horizontalHeaderItem(0).setText(QtGui.QApplication.translate("MainWindow", "Name", None, QtGui.QApplication.UnicodeUTF8))
        self.dataTable.horizontalHeaderItem(1).setText(QtGui.QApplication.translate("MainWindow", "Source", None, QtGui.QApplication.UnicodeUTF8))
        self.dataTable.horizontalHeaderItem(2).setText(QtGui.QApplication.translate("MainWindow", "Trace", None, QtGui.QApplication.UnicodeUTF8))
        self.dataTable.horizontalHeaderItem(3).setText(QtGui.QApplication.translate("MainWindow", "Comments", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("MainWindow", "Command:", None, QtGui.QApplication.UnicodeUTF8))
        self.commandButton.setText(QtGui.QApplication.translate("MainWindow", "Run", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.dataTab), QtGui.QApplication.translate("MainWindow", "&Data", None, QtGui.QApplication.UnicodeUTF8))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.plotTab), QtGui.QApplication.translate("MainWindow", "&Plot", None, QtGui.QApplication.UnicodeUTF8))
        self.menuFile.setTitle(QtGui.QApplication.translate("MainWindow", "&File", None, QtGui.QApplication.UnicodeUTF8))
        self.menuAdd_source.setTitle(QtGui.QApplication.translate("MainWindow", "&Add source", None, QtGui.QApplication.UnicodeUTF8))
        self.menuPlot.setTitle(QtGui.QApplication.translate("MainWindow", "&Graphics", None, QtGui.QApplication.UnicodeUTF8))
        self.menuCommand.setTitle(QtGui.QApplication.translate("MainWindow", "&Command", None, QtGui.QApplication.UnicodeUTF8))
        self.menuHelp.setTitle(QtGui.QApplication.translate("MainWindow", "&Help", None, QtGui.QApplication.UnicodeUTF8))
        self.actionLoadState.setText(QtGui.QApplication.translate("MainWindow", "&Load state", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSaveState.setText(QtGui.QApplication.translate("MainWindow", "&Save state", None, QtGui.QApplication.UnicodeUTF8))
        self.actionExit.setText(QtGui.QApplication.translate("MainWindow", "E&xit", None, QtGui.QApplication.UnicodeUTF8))
        self.actionExit.setToolTip(QtGui.QApplication.translate("MainWindow", "Exit pyXpad", None, QtGui.QApplication.UnicodeUTF8))
        self.actionExit.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+Q", None, QtGui.QApplication.UnicodeUTF8))
        self.actionXPAD_tree.setText(QtGui.QApplication.translate("MainWindow", "XPAD tree", None, QtGui.QApplication.UnicodeUTF8))
        self.actionXPAD_tree.setToolTip(QtGui.QApplication.translate("MainWindow", "Load a tree of XPAD items", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNetCDF_file.setText(QtGui.QApplication.translate("MainWindow", "NetCDF file", None, QtGui.QApplication.UnicodeUTF8))
        self.actionPlot.setText(QtGui.QApplication.translate("MainWindow", "&Plot", None, QtGui.QApplication.UnicodeUTF8))
        self.actionOPlot.setText(QtGui.QApplication.translate("MainWindow", "&OPlot", None, QtGui.QApplication.UnicodeUTF8))
        self.actionMPlot.setText(QtGui.QApplication.translate("MainWindow", "&MPlot", None, QtGui.QApplication.UnicodeUTF8))
        self.actionXYPlot.setText(QtGui.QApplication.translate("MainWindow", "&XYPlot", None, QtGui.QApplication.UnicodeUTF8))
        self.actionZPlot.setText(QtGui.QApplication.translate("MainWindow", "&ZPlot", None, QtGui.QApplication.UnicodeUTF8))
        self.actionBOUT_data.setText(QtGui.QApplication.translate("MainWindow", "BOUT++ data", None, QtGui.QApplication.UnicodeUTF8))
        self.actionBOUT_data.setToolTip(QtGui.QApplication.translate("MainWindow", "Read BOUT++ data directory", None, QtGui.QApplication.UnicodeUTF8))
        self.actionBOUT_data.setShortcut(QtGui.QApplication.translate("MainWindow", "Ctrl+S", None, QtGui.QApplication.UnicodeUTF8))
        self.actionContour.setText(QtGui.QApplication.translate("MainWindow", "&Contour", None, QtGui.QApplication.UnicodeUTF8))
        self.actionContour_filled.setText(QtGui.QApplication.translate("MainWindow", "Contour &filled", None, QtGui.QApplication.UnicodeUTF8))
        self.actionClearFig.setText(QtGui.QApplication.translate("MainWindow", "C&lear Figure", None, QtGui.QApplication.UnicodeUTF8))
        self.actionWrite_ASCII.setText(QtGui.QApplication.translate("MainWindow", "&Write ASCII", None, QtGui.QApplication.UnicodeUTF8))
        self.actionAdd.setText(QtGui.QApplication.translate("MainWindow", "X+Y (Sum Channels)", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSubtract.setText(QtGui.QApplication.translate("MainWindow", "X-Y", None, QtGui.QApplication.UnicodeUTF8))
        self.actionMultiply.setText(QtGui.QApplication.translate("MainWindow", "X*Y", None, QtGui.QApplication.UnicodeUTF8))
        self.actionDivide.setText(QtGui.QApplication.translate("MainWindow", "X/Y", None, QtGui.QApplication.UnicodeUTF8))
        self.actionChop.setText(QtGui.QApplication.translate("MainWindow", "Chop", None, QtGui.QApplication.UnicodeUTF8))
        self.actionIntegrate.setText(QtGui.QApplication.translate("MainWindow", "Integrate", None, QtGui.QApplication.UnicodeUTF8))
        self.actionDf_dt.setText(QtGui.QApplication.translate("MainWindow", "df/dt", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSmooth.setText(QtGui.QApplication.translate("MainWindow", "Smooth", None, QtGui.QApplication.UnicodeUTF8))
        self.actionLow_pass_filter.setText(QtGui.QApplication.translate("MainWindow", "Low pass filter", None, QtGui.QApplication.UnicodeUTF8))
        self.actionHigh_pass_filter.setText(QtGui.QApplication.translate("MainWindow", "High pass filter", None, QtGui.QApplication.UnicodeUTF8))
        self.actionBand_pass_filter.setText(QtGui.QApplication.translate("MainWindow", "Band pass filter", None, QtGui.QApplication.UnicodeUTF8))
        self.actionFFTP.setText(QtGui.QApplication.translate("MainWindow", "FFTP", None, QtGui.QApplication.UnicodeUTF8))
        self.actionRunFFT.setText(QtGui.QApplication.translate("MainWindow", "Running FFT", None, QtGui.QApplication.UnicodeUTF8))
        self.actionReciprocal.setText(QtGui.QApplication.translate("MainWindow", "1/X", None, QtGui.QApplication.UnicodeUTF8))
        self.actionExponential.setText(QtGui.QApplication.translate("MainWindow", "exp", None, QtGui.QApplication.UnicodeUTF8))
        self.actionAbsolute.setText(QtGui.QApplication.translate("MainWindow", "abs", None, QtGui.QApplication.UnicodeUTF8))
        self.actionArctan.setText(QtGui.QApplication.translate("MainWindow", "arctan", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNlog.setText(QtGui.QApplication.translate("MainWindow", "ln", None, QtGui.QApplication.UnicodeUTF8))
        self.actionNorm.setText(QtGui.QApplication.translate("MainWindow", "Normalise", None, QtGui.QApplication.UnicodeUTF8))
        self.actionInvert.setText(QtGui.QApplication.translate("MainWindow", "Invert", None, QtGui.QApplication.UnicodeUTF8))
        self.actionAddCon.setText(QtGui.QApplication.translate("MainWindow", "X+C", None, QtGui.QApplication.UnicodeUTF8))
        self.actionSubCon.setText(QtGui.QApplication.translate("MainWindow", "X-C", None, QtGui.QApplication.UnicodeUTF8))
        self.actionMulCon.setText(QtGui.QApplication.translate("MainWindow", "X*C", None, QtGui.QApplication.UnicodeUTF8))
        self.actionDivCon.setText(QtGui.QApplication.translate("MainWindow", "X/C", None, QtGui.QApplication.UnicodeUTF8))
        self.actionPowCon.setText(QtGui.QApplication.translate("MainWindow", "X^C", None, QtGui.QApplication.UnicodeUTF8))
        self.actionChangeName.setText(QtGui.QApplication.translate("MainWindow", "Change Name", None, QtGui.QApplication.UnicodeUTF8))
        self.actionChangeUnits.setText(QtGui.QApplication.translate("MainWindow", "Change Units", None, QtGui.QApplication.UnicodeUTF8))
        self.actionClip.setText(QtGui.QApplication.translate("MainWindow", "Clip", None, QtGui.QApplication.UnicodeUTF8))
        self.actionStats.setText(QtGui.QApplication.translate("MainWindow", "Statistics", None, QtGui.QApplication.UnicodeUTF8))
        self.actionTimeOff.setText(QtGui.QApplication.translate("MainWindow", "Time Offset", None, QtGui.QApplication.UnicodeUTF8))
        self.actionAbout.setText(QtGui.QApplication.translate("MainWindow", "&About", None, QtGui.QApplication.UnicodeUTF8))
        self.actionDeleteTrace.setText(QtGui.QApplication.translate("MainWindow", "&Delete Trace", None, QtGui.QApplication.UnicodeUTF8))

from .console_widget import ConsoleWidget
